"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAllCertificateEvents = exports.getAllOwnedCertificates = exports.getAllCertificates = exports.transferCertificates = exports.claimCertificates = exports.decodeClaimData = exports.encodeClaimData = void 0;
const utils_1 = require("ethers/utils");
const Certificate_1 = require("./Certificate");
const events_1 = require("../utils/events");
exports.encodeClaimData = async (claimData, configuration) => {
    const { beneficiary, address, region, zipCode, countryCode } = claimData;
    const { registry } = configuration.blockchainProperties;
    return registry.encodeClaimData(beneficiary !== null && beneficiary !== void 0 ? beneficiary : '', address !== null && address !== void 0 ? address : '', region !== null && region !== void 0 ? region : '', zipCode !== null && zipCode !== void 0 ? zipCode : '', countryCode !== null && countryCode !== void 0 ? countryCode : '');
};
exports.decodeClaimData = async (encodedClaimData, configuration) => {
    const { registry } = configuration.blockchainProperties;
    const { _beneficiary, _address, _region, _zipCode, _countryCode } = await registry.decodeClaimData(encodedClaimData);
    return {
        beneficiary: _beneficiary,
        address: _address,
        region: _region,
        zipCode: _zipCode,
        countryCode: _countryCode
    };
};
async function claimCertificates(certificateIds, claimData, configuration) {
    const certificatesPromises = certificateIds.map((certId) => new Certificate_1.Certificate(certId, configuration).sync());
    const certificates = await Promise.all(certificatesPromises);
    const ownsAllCertificates = certificates.every((cert) => cert.isOwned === true);
    if (!ownsAllCertificates) {
        throw new Error(`You can only claim your own certificates`);
    }
    const values = certificates.map((cert) => cert.energy.publicVolume);
    const encodedClaimData = await exports.encodeClaimData(claimData, configuration);
    const data = utils_1.randomBytes(32);
    const { activeUser, registry } = configuration.blockchainProperties;
    const registryWithSigner = registry.connect(activeUser);
    const activeUserAddress = await activeUser.getAddress();
    const claimTx = await registryWithSigner.safeBatchTransferAndClaimFrom(activeUserAddress, activeUserAddress, certificateIds, values, data, certificates.map(() => encodedClaimData));
    await claimTx.wait();
    return claimTx;
}
exports.claimCertificates = claimCertificates;
async function transferCertificates(certificateIds, to, configuration) {
    const certificatesPromises = certificateIds.map((certId) => new Certificate_1.Certificate(certId, configuration).sync());
    const certificates = await Promise.all(certificatesPromises);
    const ownsAllCertificates = certificates.every((cert) => cert.isOwned === true);
    if (!ownsAllCertificates) {
        throw new Error(`You can only claim your own certificates`);
    }
    const values = certificates.map((cert) => cert.energy.publicVolume);
    // TO-DO: replace with proper data
    const data = utils_1.randomBytes(32);
    const { activeUser } = configuration.blockchainProperties;
    const { registry } = configuration.blockchainProperties;
    const registryWithSigner = registry.connect(activeUser);
    const activeUserAddress = await activeUser.getAddress();
    const transferTx = await registryWithSigner.safeBatchTransferFrom(activeUserAddress, to, certificateIds, values, data);
    await transferTx.wait();
    return transferTx;
}
exports.transferCertificates = transferCertificates;
async function getAllCertificates(configuration) {
    const { issuer } = configuration.blockchainProperties;
    const certificationRequestApprovedEvents = await events_1.getEventsFromContract(issuer, issuer.filters.CertificationRequestApproved(null, null, null));
    const certificatePromises = certificationRequestApprovedEvents.map((event) => new Certificate_1.Certificate(event.values._certificateId.toNumber(), configuration).sync());
    return Promise.all(certificatePromises);
}
exports.getAllCertificates = getAllCertificates;
async function getAllOwnedCertificates(configuration) {
    const { registry, activeUser } = configuration.blockchainProperties;
    const owner = await activeUser.getAddress();
    const transfers = await events_1.getEventsFromContract(registry, registry.filters.TransferSingle(null, null, owner, null, null));
    const certificateIds = [
        ...new Set(transfers.map((transfer) => transfer.values._id.toNumber()))
    ];
    const balances = await registry.balanceOfBatch(Array(certificateIds.length).fill(owner), certificateIds);
    const available = certificateIds.filter((id, index) => !balances[index].isZero());
    const certificatePromises = available.map((id) => new Certificate_1.Certificate(id, configuration).sync());
    return Promise.all(certificatePromises);
}
exports.getAllOwnedCertificates = getAllOwnedCertificates;
exports.getAllCertificateEvents = async (certId, configuration) => {
    const { registry } = configuration.blockchainProperties;
    const getEvent = async (filter, eventName) => {
        const logs = await registry.provider.getLogs({
            ...filter,
            fromBlock: 0,
            toBlock: 'latest'
        });
        const parsedLogs = await Promise.all(logs.map(async (event) => {
            const { values } = registry.interface.parseLog(event);
            const eventBlock = await registry.provider.getBlock(event.blockHash);
            return {
                name: eventName,
                transactionHash: event.transactionHash,
                blockHash: event.blockHash,
                values,
                timestamp: eventBlock.timestamp
            };
        }));
        return parsedLogs.filter((event) => event.values._id.toNumber() === certId);
    };
    const issuanceSingleEvents = await getEvent(registry.filters.IssuanceSingle(null, null, null), 'IssuanceSingle');
    const transferSingleEvents = await getEvent(registry.filters.TransferSingle(null, null, null, null, null), 'TransferSingle');
    const claimSingleEvents = await getEvent(registry.filters.ClaimSingle(null, null, null, null, null, null), 'ClaimSingle');
    return [...issuanceSingleEvents, ...transferSingleEvents, ...claimSingleEvents];
};
//# sourceMappingURL=CertificateUtils.js.map